#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Simple Weather example
# This file is part of the Librem 5 developer documentation.

# Copyright (C) 2019 Purism SPC
# SPDX-License-Identifier: GPL-3.0+
# Author: David Boddie <david.boddie@puri.sm>

import cairo
import gettext
import gi
import glob
import gzip
import os
import sys

gi.require_version('Gtk', '3.0')
gi.require_version('Poppler', '0.18')

from gi.repository import GLib, Gio, Gtk, Poppler

# Placeholder to be replaced by the build system
localedir = '@localedir@'
prefix = '@prefix@'
document_pattern = os.path.join(prefix, 'share', 'doc',
                                'librem5-quick-start-guide',
                                'Librem5-Quick-Start.pdf*')

class Application(Gtk.Application):

    def __init__(self, doc):
        super().__init__(application_id='sm.puri.QuickStartViewer')
        GLib.set_application_name(_('Quick Start Guide'))
        GLib.set_prgname('sm.puri.QuickStartViewer')

        self.doc = doc
        self.index = 0

    def do_startup(self):
        Gtk.Application.do_startup(self)

        self.font_options = cairo.FontOptions()
        self.font_options.set_antialias(cairo.ANTIALIAS_BEST)

    def do_activate(self):
        window = Gtk.ApplicationWindow(application=self)

        # Title bar and navigation buttons

        header = Gtk.HeaderBar(title=_('Quick Start Guide'),
                               show_close_button=True)

        self.previous_button = Gtk.Button.new_from_icon_name('go-previous-symbolic', Gtk.IconSize.MENU)
        self.next_button = Gtk.Button.new_from_icon_name('go-next-symbolic', Gtk.IconSize.MENU)

        header.add(self.previous_button)
        header.add(self.next_button)

        window.set_titlebar(header)

        # Main content area

        self.scroller = Gtk.ScrolledWindow(
            hexpand=True,
            kinetic_scrolling=True,
            vexpand=True
        )

        self.page_widget = Gtk.Image()
        self.scroller.add(self.page_widget)

        self.previous_button.connect('clicked', self.go_previous)
        self.next_button.connect('clicked', self.go_next)
        window.connect('size-allocate', self.draw_page)

        window.add(self.scroller)
        window.set_default_size(360, 720)
        window.show_all()

        self.update_buttons()

    def draw_page(self, *args):

        page = self.doc.get_page(self.index)

        rect = self.scroller.get_allocation()
        sw, sh = rect.width, rect.height
        w, h = page.get_size()

        # Fit the width of the page in the scroller.
        sh = int((h / w) * sw)
        surface = cairo.ImageSurface(cairo.FORMAT_RGB24, sw, sh)

        context = cairo.Context(surface)
        context.set_antialias(cairo.ANTIALIAS_BEST)
        context.set_font_options(self.font_options)

        # Paint the background color.
        context.set_source_rgb(1, 1, 1)
        context.paint()

        # Make the page fit the available size.
        scale = min(sw / w, sh / h)
        x = (sw - (scale * w)) / 2
        y = (sh - (scale * h)) / 2
        context.translate(x, y)
        context.scale(scale, scale)

        # Render the PDF to the surface using the context.
        page.render(context)
        # Use the surface in the image widget.
        self.page_widget.set_from_surface(surface)

    def update_buttons(self):

        self.previous_button.set_sensitive(self.index > 0)
        self.next_button.set_sensitive(self.index < self.doc.get_n_pages() - 1)

    def go_previous(self, button):

        self.index -= 1
        self.draw_page()
        self.update_buttons()

    def go_next(self, button):

        self.index += 1
        self.draw_page()
        self.update_buttons()


if __name__ == '__main__':

    for path in glob.glob(document_pattern):
        if path.endswith('.gz'):
            data = gzip.GzipFile(path).read()
            break
        elif path.endswith('.pdf'):
            data = open(path, 'rb').read()
            break
    else:
        sys.stderr.write('Failed to find the Quick Start Guide.\n')
        sys.exit(1)

    # https://stackoverflow.com/questions/42735374/has-anyone-been-able-to-use-poppler-new-from-data-in-python
    input_stream = Gio.MemoryInputStream.new_from_data(data)
    doc = Poppler.Document.new_from_stream(input_stream, -1)

    # Install a translator for Python code to use throughout the application.
    gettext.install('librem5-quick-start', localedir, names=['ngettext'])

    app = Application(doc)
    sys.exit(app.run(sys.argv))
